#! perl -w

# Licensed Materials - Property of IBM 
# (c) Copyright IBM Corporation 1991, 2005 All Rights Reserved. 

# rtlink.pl - Link controller

# Unbuffer stdout

select STDERR; $| = 1;
select STDOUT; $| = 1;

# load in the error handler

$codegen = $0;
$codegen =~ s{\\}{/}g;
$codegen =~ s{/rtlink.pl$}{};

require "$codegen/rterror.pl";

# Silence perl: other uses are in rterror.pl.
$rterror::maxError = $rterror::maxError;

# Get the arguments

$component  = '';
$isLib      = 0;
$isPurify   = 0;
$isQuantify = 0;
$isPurecov  = 0;
$targetname = '';

while( $#ARGV >= 0 )
{
    $arg = shift @ARGV;
    if   ( $arg eq '-vendor'    ) { $vendor     = shift @ARGV; }
    elsif( $arg eq '-purify'    ) { $isPurify   = 1; }
	elsif( $arg eq '-quantify'  ) { $isQuantify   = 1; }
	elsif( $arg eq '-coverage'  ) { $isPurecov   = 1; }
    elsif( $arg eq '-exe'       ) { $targetname = shift @ARGV; $isLib = 0; }
    elsif( $arg eq '-lib'       ) { $targetname = shift @ARGV; $isLib = 1; }
    elsif( $arg eq '-component' ) { $component  = shift @ARGV; }
    elsif( $arg eq '--'         ) { last; }
    else
    {
	&rterror::action_message( "Unknown rtlink.pl option $arg", 1 );
    }
}

%config = (
	    'exename'       => '',
	    'purifyexename' => '',
		'quantifyexename' => '',
		'coverageexename' => ''
	  );

# Read the executable name from the rtbuild.dat file.

if( open DAT, '../rtbuild.dat' )
{
    while( <DAT> )
    {
	chomp;
	( $key, $value ) = split( '=' );
	$config{$key} = $value;
    }
    close DAT;
}

if( ( $isPurify ? $config{purifyexename}
		: $config{exename}       ) ne $targetname )
{
    if( $targetname =~ /^\// || $targetname =~ /^\\/ || $targetname =~ /^.:/ )
    {
	$exepath = $targetname;
    }
    else
    {
	$exepath = 'build/' . $targetname;
    }

    if( $isPurify )
    {
	$config{purifyexename} = $exepath;
    }
	elsif ($isQuantify )
    {
	$config{quantifyexename} = $exepath;
    }
	elsif ($isPurecov )
    {
	$config{coverageexename} = $exepath;
    }
    else
    {
	$config{exename} = $exepath;
    }

    if( open DAT, '>../rtbuild.dat' )
    {
	foreach $key ( keys %config )
	{
	    print DAT $key, '=', $config{$key}, "\n";
	}
	close DAT;
    }
}

# load in configs (for now, just the compiler)

require "$codegen/compiler/$vendor/link.pl";

if( $isLib )
{
    print STDERR "!> Building library $targetname\n";
}
else
{
    print STDERR "!> Linking $targetname\n";
}

$command = '';
foreach $arg ( @ARGV )
{
    if( $arg =~ / / )
    {
	$arg = '"' . $arg . '"';
    }
    if( $command )
    {
	$command .= ' ';
    }
    $command .= $arg;
}

$OS          = '?' unless $OS          = $ENV{'OS'};
$ROSERT_HOST = '?' unless $ROSERT_HOST = $ENV{'ROSERT_HOST'};

$errRedirect = ( $ROSERT_HOST eq 'win32' && $OS !~ m{^Windows_NT$}i )
			? ''
			: '2>&1';

open OPEN, "$command $errRedirect |";

@errors = <OPEN>;

close OPEN;
$retcode = $?;

print @errors;
&rterror::handle_errors();

# now handle return code from command

if( $rterror::worstError < $rterror::maxError )
{
    if( $rterror::worstError == 0 )
    {
	exit 1;
    }
}
elsif( $retcode & 0xff )
{
    &rterror::action_message( "caught signal [$retcode] from [$command]", 0 );
    exit 1;
}
elsif( $retcode )
{
    $retcode >>= 8;
    &rterror::action_message( "detected error [$retcode] from [$command]", 0 );
    exit 1;
}
