/* 
 *  Licensed Materials - Property of IBM 
 *  Component IBM Rational ROSE_RT BUILD : TARGET_RTS 
 *  (c) Copyright IBM Corporation 1991, 2005 All Rights Reserved. 
 */

#ifndef __RTAsciiDecoding_h__
#define __RTAsciiDecoding_h__ included

#ifdef PRAGMA
#pragma interface
#endif

#ifndef __RTDecoding_h__
#include <RTDecoding.h>
#endif

#if OBJECT_DECODE

class  RTDataObject;
class  RTIBuffer;
struct RTUnknownObject;

class RTAsciiDecoding : public RTDecoding
{
public:
    explicit    RTAsciiDecoding( RTIBuffer * );
    virtual     ~RTAsciiDecoding( void );

    // All the get/put functions return an int which is non-zero
    // if the operation was successful. Otherwise, the following
    // function may be called to discover the reason for the
    // failure.
    int         getError( void );

    virtual int get( void **, const RTObject_class ** );

    // basic data types
    virtual int get_address ( void         * & );
    virtual int get_bool    ( bool           & );
    virtual int get_char    ( char           & );
#if RTUseFloatingPoint
    virtual int get_double  ( double         & );
    virtual int get_float   ( float          & );
#endif
    virtual int get_int     ( int            & );
    virtual int get_long    ( long           & );

	virtual int get_unsignedint( unsigned int & );
	virtual int get_unsignedlong( unsigned long & );
	virtual int get_unsignedshort( unsigned short & );
	virtual int get_unsignedchar( unsigned char & );
#if 0
#if _MSC_VER > 1000
    virtual int get_long_long( __int64 &  );
#else
    virtual int get_long_long( long long &  );
#endif
#endif

    virtual int get_short   ( short          & );
    virtual int get_uchar   ( unsigned char  & );
    virtual int get_unsigned( unsigned       & );
    virtual int get_ulong   ( unsigned long  & );
    virtual int get_ushort  ( unsigned short & );

    // a nul-terminated string
    virtual int get_string( char * & );

    // zero or more raw bytes
    virtual int get_opaque( int &, char * & );

    // an enumerated value
    virtual int get_enum( int &, int, const RTFieldDescriptor * );

    // a homogeneous array of objects
    virtual int get_array( void *, int, const RTObject_class * );

    // an indirect object
    virtual int get_indirect( void *, const RTObject_class * );

    // a structure
    virtual int get_struct( void *, const RTObject_class * );

protected:
    int                 getName( char * value, int length );
    RTUnknownObject   * getUnknown( const char * );

    int                 getNonWhite( char & );
    int                 getByte( char & );
    void                ungetByte( char );

    int                 getFieldListStart( void );
    int                 getFieldName( char * value, int length );
    int                 getFieldSeparator( void );
    int                 getFieldListFinish( void );

private:
    enum              { UngetMax = 3 };

    int                 unget_size;
    char                unget_buffer[ UngetMax ];
    RTIBuffer         * input;

    // unavailable methods
    RTAsciiDecoding( const RTAsciiDecoding & );
    RTAsciiDecoding & operator=( const RTAsciiDecoding & );
}; //lint !e1712

#endif // OBJECT_DECODE

#endif // __RTAsciiDecoding_h__
