/* 
 *  Licensed Materials - Property of IBM 
 *  Component IBM Rational ROSE_RT BUILD : TARGET_RTS 
 *  (c) Copyright IBM Corporation 1991, 2005 All Rights Reserved. 
 */

#ifndef __RTAsciiEncoding_h__
#define __RTAsciiEncoding_h__ included

#ifdef PRAGMA
#pragma interface
#endif

#ifndef __RTEncoding_h__
#include <RTEncoding.h>
#endif

#if OBJECT_ENCODE

class RTOBuffer;

#if OBJECT_DECODE && ! defined( __RTAsciiDecoding_h__ )
#include <RTAsciiDecoding.h>
#endif

class RTAsciiEncoding : public RTEncoding
{
public:
    explicit    RTAsciiEncoding( RTOBuffer * );
    virtual     ~RTAsciiEncoding( void );

    // All the get/put functions return an int which is non-zero
    // if the operation was successful. Otherwise, the following
    // function may be called to discover the reason for the
    // failure.
    int         getError( void );

    virtual int flush( void );

    virtual int put( const void *, const RTObject_class * );

    // Functions which deal with the basic data types.
    virtual int put_address ( const void   * );
    virtual int put_bool    ( bool           );
    virtual int put_char    ( char           );
#if RTUseFloatingPoint
    virtual int put_double  ( double         );
    virtual int put_float   ( float          );
#endif
    virtual int put_int     ( int            );
    virtual int put_long    ( long           );
    virtual int put_short   ( short          );
    virtual int put_uchar   ( unsigned char  );
    virtual int put_unsigned( unsigned       );
    virtual int put_ulong   ( unsigned long  );
    virtual int put_ushort  ( unsigned short );

    virtual int put_unsignedint( unsigned int);
    virtual int put_unsignedlong( unsigned long);
	virtual int put_unsignedshort( unsigned short);
	virtual int put_unsignedchar( unsigned char);
    virtual int put_string( const char * );

    virtual int put_opaque( int, const char * );

    virtual int put_enum( int, int, const RTFieldDescriptor * );

    virtual int put_array( const void *, int, const RTObject_class * );

    virtual int put_indirect( const void *, const RTObject_class * );

    virtual int put_struct( const void *, const RTObject_class * );

protected:
    enum Spacing { SpaceNone, SpaceAround };

    RTOBuffer     * output;
    Spacing         before;

    virtual int     putType( const RTObject_class * );
    int             putName( const char * );

    int             putFields( const void           *,
			       const RTObject_class *,
			       int                  & first );

    int             putFieldListStart( void );
    int             putFieldSeparator( void );
    int             putFieldListFinish( void );

    int             putCode( const char *, int, Spacing );

private:
    // unavailable methods
    RTAsciiEncoding( const RTAsciiEncoding & );
    RTAsciiEncoding & operator=( const RTAsciiEncoding & );
}; //lint !e1712

#endif // OBJECT_ENCODE

#endif // __RTAsciiEncoding_h__
