/* 
 *  Licensed Materials - Property of IBM 
 *  Component IBM Rational ROSE_RT BUILD : TARGET_RTS 
 *  (c) Copyright IBM Corporation 1991, 2005 All Rights Reserved. 
 */

#ifndef __RTEncoding_h__
#define __RTEncoding_h__ included

#ifdef PRAGMA
#pragma interface
#endif

#ifndef __RTConfig_h__
#include <RTConfig.h>
#endif

#if OBJECT_ENCODE

struct RTFieldDescriptor;
struct RTObject_class;

class RTEncoding
{
public:
		RTEncoding( void );
    virtual     ~RTEncoding( void );

    virtual int flush( void ) = 0;

    virtual int put( const void *, const RTObject_class * ) = 0;

    // basic data types
    virtual int put_address ( const void   * ) = 0;
    virtual int put_bool    ( bool           ) = 0;
    virtual int put_char    ( char           ) = 0;
#if RTUseFloatingPoint
    virtual int put_double  ( double         ) = 0;
    virtual int put_float   ( float          ) = 0;
#endif
    virtual int put_int     ( int            ) = 0;
    virtual int put_long    ( long           ) = 0;
    virtual int put_short   ( short          ) = 0;
    virtual int put_uchar   ( unsigned char  ) = 0;
    virtual int put_unsigned( unsigned       ) = 0;
    virtual int put_ulong   ( unsigned long  ) = 0;
    virtual int put_ushort  ( unsigned short ) = 0;
    virtual int put_unsignedint( unsigned int ) = 0;
    virtual int put_unsignedlong( unsigned long ) = 0;
	virtual int put_unsignedshort( unsigned short ) = 0;
	virtual int put_unsignedchar( unsigned char ) = 0;

    // a nul-terminated string
    virtual int put_string( const char * ) = 0;

    // zero or more raw bytes
    virtual int put_opaque( int, const char * ) = 0;

    // an enumerated value
    virtual int put_enum( int                       value,
			  int                       numChoices,
			  const RTFieldDescriptor * choices ) = 0;

    // a homogeneous array of objects
    virtual int put_array( const void           * array_base,
			   int                    numElements,
			   const RTObject_class * elementType ) = 0;

    // an indirect object
    virtual int put_indirect( const void           * pointer,
			      const RTObject_class * targetType ) = 0;

    // a structure
    virtual int put_struct( const void           * record,
			    const RTObject_class * recordType ) = 0;

private:
    // unavailable methods
    RTEncoding( const RTEncoding & );
    RTEncoding & operator=( const RTEncoding & );
};
#endif // OBJECT_ENCODE

#endif // __RTEncoding_h__
