/* 
 *  Licensed Materials - Property of IBM 
 *  Component IBM Rational ROSE_RT BUILD : TARGET_RTS 
 *  (c) Copyright IBM Corporation 1991, 2005 All Rights Reserved. 
 */

#include <RTActor.h>
#include <RTActorProbe.h>
#include <RTController.h>
#include <RTDebugger.h>
#include <RTEventInfo.h>
#include <RTLayerConnector.h>
#include <RTMessage.h>
#include <RTPortDescriptor.h>
#include <RTProtocol.h>

int RTProtocol::reply( int                    signal,
		       const void           * data,
		       const RTObject_class * type )
{
    RTActor         * owner      = getOwner();
    RTController    * controller = owner->context();
    const RTMessage * msg        = owner->msg;
    int               index      = msg->sapIndex0();

    controller->trace( RTController::CommReply, this, &index );

    if( this != msg->sap() )
    {
	controller->setError( RTController::badOperation,
			      RTController::CommReply, this, &index );
	return 0;
    }

#if RTTYPECHECK_SEND != RTTYPECHECK_DONT
    switch( sendCheck( signal, type ) )
    {
    case RTProtocolDescriptor::badSignal:
	controller->setError( RTController::badSignal,
			      RTController::CommReply, this, &index );
#if RTTYPECHECK_SEND == RTTYPECHECK_FAIL
	return 0;
#else
	break;
#endif

    case RTProtocolDescriptor::badType:
	controller->setError( RTController::badClass,
			      RTController::CommReply, this, &index );
#if RTTYPECHECK_SEND == RTTYPECHECK_FAIL
	return 0;
#else
	break;
#endif

    case RTProtocolDescriptor::ok:
    default:
	break;
    }
#endif // RTTYPECHECK_SEND != RTTYPECHECK_DONT

    End        * end         = &_ends[ index ];
    RTMessage  * replyBuffer = controller->getReplyBuffer();
    RTProtocol * remotePort;
    int          remoteIndex;
    RTActor    * remoteActor;
    int          inSignal;

    if( end->signalMap != (const int *)0 )
	inSignal = end->signalMap[ signal - 1 ];
    else
	inSignal = signal;

    // check to see if this is a reply to an invoked message
	if( replyBuffer == (RTMessage *)0 )
	{
		replyBuffer = controller->newMsg();

		if( replyBuffer == (RTMessage *)0 )
		{
			controller->setError( RTController::noMem,
								  RTController::CommReply, this, &index );
			return 0;
		}

		replyBuffer->setData( data, type );
		replyBuffer->setSignal( inSignal );
		replyBuffer->setPriority( msg->getPriority() );

#if OBSERVABLE
		remotePort  = end->remote;
		remoteIndex = end->remoteIndex;
		remoteActor = end->state != End::Unbound ? remotePort->getOwner()
												 : (RTActor *)0;

		replyBuffer->setDestination( remoteActor, remotePort, remoteIndex );

		controller->debugger()->setTimestamp( replyBuffer, this, index );

		if( owner->probe != (RTActorProbe *)0 )
			owner->probe->evtMessageOut( replyBuffer, RTEventInfo::SendReply );
#endif

		if( end->state == End::Unbound )
		{
			controller->setError( RTController::noConnect,
								  RTController::CommReply, this, &index );
			controller->freeMsg( replyBuffer );
			return 0;
		}

		replyBuffer->internData();

#if USE_THREADS
		// The binding may have been severed while we were busy interning the data.
		int need_lock = 0;

		switch( end->state )
		{
		case End::Unbound:
			controller->setError( RTController::noConnect,
								  RTController::CommReply, this, &index );
			controller->freeMsg( replyBuffer );
			return 0;

		case End::BoundXThread:
			need_lock = 1;
			RTLayerConnector::lock();
			break;

		default:
			break;
		}
#endif

		remotePort  = end->remote;
		remoteIndex = end->remoteIndex;
		remoteActor = remotePort->getOwner();

		replyBuffer->setDestination( remoteActor, remotePort, remoteIndex );

#if USE_THREADS
	replyBuffer->context()->receive( replyBuffer, controller );
#else
	controller->receive( replyBuffer, controller );
#endif

#if USE_THREADS
	if( need_lock != 0 )
	    RTLayerConnector::unlock();
#endif

	return 1;
    }

    if( end->state == End::Unbound )
    {
	controller->setError( RTController::noConnect,
			      RTController::CommReply, this, &index );
	return 0;
    }

    remotePort  = end->remote;
    remoteIndex = end->remoteIndex;
    remoteActor = remotePort->getOwner();

#if RTTYPECHECK_RECEIVE != RTTYPECHECK_DONT
    switch( remotePort->receiveCheck( inSignal, type ) )
    {
    case RTProtocolDescriptor::badSignal:
	controller->setError( RTController::badSignal,
			      RTController::CommReply, this, &index );
#if RTTYPECHECK_RECEIVE == RTTYPECHECK_FAIL
	return 0;
#else
	break;
#endif

    case RTProtocolDescriptor::badType:
	controller->setError( RTController::badClass,
			      RTController::CommReply, this, &index );
#if RTTYPECHECK_RECEIVE == RTTYPECHECK_FAIL
	return 0;
#else
	break;
#endif

    case RTProtocolDescriptor::ok:
    default:
	break;
    }
#endif // RTTYPECHECK_RECEIVE != RTTYPECHECK_DONT

    replyBuffer->clear();
    replyBuffer->setSignal( inSignal );
    replyBuffer->setData( data, type );
    replyBuffer->internData();
    replyBuffer->setDestination( remoteActor, remotePort, remoteIndex );

#if OBSERVABLE
    controller->debugger()->setTimestamp( replyBuffer, this, index );

    if( owner->probe != (RTActorProbe *)0 )
    {
	owner->probe->evtMessageOut( replyBuffer, RTEventInfo::SendReply );

	if( remoteActor->probe != (RTActorProbe *)0 )
	    remoteActor->probe->evtMessageOutIn( replyBuffer,
						 RTEventInfo::SendReply );
    }
#endif

    return 1;
}
