/* 
 *  Licensed Materials - Property of IBM 
 *  Component IBM Rational ROSE_RT BUILD : TARGET_RTS 
 *  (c) Copyright IBM Corporation 1991, 2005 All Rights Reserved. 
 */

#include <RTActor.h>
#include <RTActorProbe.h>
#include <RTController.h>
#include <RTDebugger.h>
#include <RTLayerConnector.h>
#include <RTMessage.h>
#include <RTProtocol.h>

int RTProtocol::End::send( RTController         * sender,
			   RTProtocol           * port,
			   int                    index,
			   int                    signal,
			   const void           * data,
			   const RTObject_class * type,
			   int                    prio )
{
    RTMessage * msg = sender->newMsg();

    if( msg == (RTMessage *)0 )
    {
	sender->setError( RTController::noMem,
			  RTController::CommSend, port, &index );
	return 0;
    }

    msg->setData( data, type );
    msg->setPriority( prio );

    if( signalMap != (const int *)0 )
	msg->setSignal( signalMap[ signal - 1 ] );
    else
	msg->setSignal( signal );

    RTActor * remoteActor;

#if OBSERVABLE
    remoteActor = state != Unbound ? remote->getOwner()
				   : (RTActor *)0;

    msg->setDestination( remoteActor, remote, remoteIndex );

    sender->debugger()->setTimestamp( msg, port, index );

    RTActor * owner = port->getOwner();

    if( owner->probe != (RTActorProbe *)0 )
	owner->probe->evtMessageOut( msg, RTEventInfo::SendNormal );
#endif

    if( state == Unbound )
    {
	sender->setError( RTController::noConnect,
			  RTController::CommSend, port, &index );

	sender->freeMsg( msg );

	return 0;
    }
    
    msg->internData();

#if USE_THREADS
	// The binding may have been severed while we were busy interning the data.
	int need_lock = 0;

	switch( state )
	{
	case Unbound:
		sender->setError( RTController::noConnect,
						  RTController::CommSend, port, &index );
		sender->freeMsg( msg );
		return 0;

	case BoundXThread:
		need_lock = 1;
		RTLayerConnector::lock();
		break;

	default:
		break;
	}
#endif

    remoteActor = remote->getOwner();
    msg->setDestination( remoteActor, remote, remoteIndex );

#if USE_THREADS
    remoteActor->context()->receive( msg, sender );
#else
    sender->receive( msg, sender );
#endif

#if USE_THREADS
    if( need_lock != 0 )
	RTLayerConnector::unlock();
#endif

    return 1;
}
